import datetime
import logging
import tarfile
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import List, Type

from base_installer.task_runner.base_task_runner import BaseTaskRunner
from collect.configuration.collect_configuration import CollectConfiguration
from collect.tasks.collect_host_files_task import CollectHostFilesTask
from collect.tasks.collect_shell_output_task import CollectShellOutputTask
from collect.tasks.collect_task import CollectTask
from collect.tasks.copy_configs_task import CopyConfigsTask
from collect.tasks.copy_container_dirs_task import CopyContainerDirsTask
from collect.tasks.copy_container_files_task import CopyContainerFilesTask
from collect.tasks.copy_logs_task import CopyLogsTask
from collect.tasks.download_siem_logs_task import DownloadSiemLogsTask
from collect.tasks.dump_dns_task import DumpDnsTask
from collect.tasks.dump_etcd_task import DumpEtcdTask
from collect.tasks.dump_kafka_task import DumpKafkaTopicsTask
from collect.tasks.dump_postgres_task import DumpPostgresTask
from collect.tasks.dump_prometheus_task import DumpPrometheusTask
from collect.tasks.dump_redis_task import DumpRedisTask
from collect.tasks.get_ceph_diagnostic_task import GetCephDiagnosticTask
from collect.tasks.get_docker_diagnostic_task import GetDockerDiagnosticTask
from collect.tasks.get_network_configuration_logs_task import GetNetworkConfigurationLogs
from collect.tasks.get_swarm_diagnostic_task import GetSwarmDiagnosticTask
from collect.tasks.show_warning_task import ShowWarningTask


class Collector(BaseTaskRunner):

    TASKS: List[Type[CollectTask]] = [
        CopyConfigsTask,
        CopyLogsTask,
        DumpDnsTask,
        DumpEtcdTask,
        GetCephDiagnosticTask,
        GetDockerDiagnosticTask,
        GetSwarmDiagnosticTask,
        DownloadSiemLogsTask,
        GetNetworkConfigurationLogs,
        DumpPrometheusTask,
        CollectHostFilesTask,
        CollectShellOutputTask,
        CopyContainerFilesTask,
        CopyContainerDirsTask,
        DumpPostgresTask,
        DumpRedisTask,
        DumpKafkaTopicsTask,
        ShowWarningTask,
    ]

    def __init__(self, configuration: CollectConfiguration):
        self._configuration = configuration

    def run(self):
        with TemporaryDirectory() as temp_dir:
            temp_dir_path = Path(temp_dir)
            self._configuration.work_dir = temp_dir_path
            super().run()
            self._make_tarfile(temp_dir_path)

    def _make_tarfile(self, src_dir: Path):
        logger = logging.getLogger(__name__)
        output_collect_dir = self._configuration.output_collect_dir
        output_collect_dir.mkdir(parents=True, exist_ok=True)
        archive_name = \
            f'collect-{self._configuration.hostname}-{datetime.datetime.utcnow().strftime("%Y-%m-%dT%H-%M-%S")}'

        logger.info('Archiving collected data...')
        with tarfile.open(output_collect_dir / f'{archive_name}.tar.gz', 'w:gz') as tar:
            tar.add(
                src_dir,
                recursive=True,
                arcname=archive_name
            )
        logger.info(f'All done: /tmp/collect/{archive_name}')
