#!/usr/bin/python3.8
import argparse
import logging
from datetime import datetime
from pathlib import Path

from collect.constants import LogNames, SiemProxyApiInfo
from collect.helpers.logs_downloader import LogsDownloader


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser('collect SIEM logs args')
    parser.add_argument('log_name', help='log name to download', choices=list(map(lambda n: n.value, LogNames)))
    parser.add_argument('--range-start', type=datetime.fromisoformat,
                        help='start datetime to fetch logs for (ISO format 2022-11-11 2022-12-12T15:15:15])')
    parser.add_argument('--range-end', type=datetime.fromisoformat,
                        help='end date to fetch logs for (ISO format 2022-11-11 2022-12-12T15:15:15])')
    parser.add_argument('--output-dir', type=Path, default=Path('/tmp/collect/logs/kaspersky/siem'))
    parser.add_argument('--no-compress', action='store_true')
    parser.add_argument('--internal', help='for use by other services in stack', action='store_true')
    args = parser.parse_args()

    if args.output_dir:
        args.output_dir.mkdir(parents=True, exist_ok=True)

    if args.no_compress:
        logging.getLogger(__file__).warning(
            'WARNING: uncompressed logs may be very large and downloads may take more than 10 minutes.'
        )

    return parser.parse_args()


def get_logs_downloader(args: argparse.Namespace) -> LogsDownloader:
    if args.internal:
        return LogsDownloader(
            host=SiemProxyApiInfo.internal_connection_host,
            port=SiemProxyApiInfo.internal_connection_port,
        )
    return LogsDownloader(
        host=SiemProxyApiInfo.external_connection_host,
        port=SiemProxyApiInfo.external_connection_port,
    )


def main():
    args = parse_args()
    ext = 'log' if args.no_compress else 'zip'

    dl = get_logs_downloader(args)
    dl.download(
        log_name=args.log_name,
        target_file=args.output_dir / f'{args.log_name}.{ext}',
        compressed=not args.no_compress,
        range_start=args.range_start,
        range_end=args.range_end,
    )
