from datetime import datetime
from functools import partial
from pathlib import Path

import requests


class LogsDownloader:
    API_ENDPOINT = '/logs/download'

    def __init__(self, host: str, port: int, timeout: int = 60):
        self._request = partial(
            requests.post,
            url=f'http://{host}:{port}{self.API_ENDPOINT}',
            timeout=timeout,
        )

    def download(self,
                 log_name: str,
                 target_file: Path,
                 compressed: bool = True,
                 range_start: datetime = None,
                 range_end: datetime = None) -> None:
        params = {
            'log_name': log_name,
            'compressed': compressed,
        }
        if range_start:
            params['range_start'] = self._format_range_date(range_start)
        if range_end:
            params['range_end'] = self._format_range_date(range_end)

        response = self._request(
            json=params,
            stream=True,
            timeout=15 * 60,  # NOTE default timeout is not enough to download large uncompressed logs
        )
        response.raise_for_status()

        with open(target_file, 'wb') as f:
            for chunk in response.iter_content(chunk_size=None):
                f.write(chunk)

    @staticmethod
    def _format_range_date(range_date: datetime) -> str:
        return f'{range_date.year}-{range_date.month}-{range_date.day}-{range_date.hour}'
