import itertools
import shlex
import shutil
from distutils.dir_util import copy_tree
from pathlib import Path
from typing import Iterable

from base_installer.tasks.helpers.subprocess_runner import SubprocessRunner
from docker_utils import docker_exec, docker_exec_with_output
from docker_utils.container import get_containers_id

__all__ = [
    'collect_env_file',
    'container_chmod',
    'copy_from_container',
    'copy_to_container',
    'db_exists',
    'docker_exec_redis_command',
    'docker_exec_sql_command',
    'docker_exec_sql_script',
    'dump_shell_command',
    'ensure_container_exists',
    'read_write_env_file',
    'stream_file_from_container',
    'stream_dir_from_container',
]


def stream_file_from_container(container: str, src: str, dst: str) -> None:
    container_id = ensure_container_exists(container)
    cmd = shlex.split(f'exec {container_id} cat "{src}"')
    with open(dst, 'w') as f:
        docker_exec(cmd, stdout=f)


def stream_dir_from_container(container: str, src: str, dst: str) -> None:
    container_id = ensure_container_exists(container)
    tmp_archive = Path('/tmp') / dst.split('/')[-1]
    tar_command = ['exec', container_id, 'bash', '-c', f'tar -czvf {tmp_archive} {src}']
    docker_exec(tar_command)
    cmd = shlex.split(f'exec {container_id} cat "{tmp_archive}"')
    with open(dst, 'w') as f:
        docker_exec(cmd, stdout=f)
    docker_exec(['exec', container_id, 'bash', '-c', f'rm -rf {tmp_archive}'])


def copy_from_container(container: str, src: str, dst: str) -> None:
    container_id = ensure_container_exists(container)
    cmd = shlex.split(f'cp {container_id}:"{src}" "{dst}"')
    docker_exec(cmd)


def copy_to_container(container: str, src: str, dst: str) -> None:
    container_id = ensure_container_exists(container)
    cmd = shlex.split(f'cp "{src}" {container_id}:"{dst}"')
    docker_exec(cmd)


def container_chmod(container: str, path: str, mode: int) -> None:
    container_id = ensure_container_exists(container)
    cmd = shlex.split(f'-it {container_id} {oct(mode).replace("o", "")} "{path}"')
    docker_exec(cmd)


def collect_env_file(src: str, dst: str):
    if Path(src).is_file():
        shutil.copy2(src, dst)
    else:
        Path(dst).mkdir(exist_ok=True, parents=True)
        copy_tree(src, dst)


def dump_shell_command(cmd: str, output: str = None) -> str:
    result = SubprocessRunner.run(cmd, shell=True)
    if output:
        Path(output).write_text(result)
    return result


def read_write_env_file(src: str, dst: str) -> None:
    Path(dst).write_text(Path(src).read_text())


def docker_exec_sql_script(container: str, db_name: str, script_path: str, output: str = None):
    container_id = ensure_container_exists(container)
    cmd = shlex.split(f'-i -u postgres {container_id} psql -U postgres -d {db_name} -f "{script_path}"')
    if output is None:
        docker_exec_with_output(cmd)
    else:
        with open(output, 'w') as f:
            docker_exec(cmd, stdout=f)


def docker_exec_sql_command(container: str, db_name: str, command: str, output: str = None):
    container_id = ensure_container_exists(container)
    sql_command = list(_interleave(itertools.repeat('-c'), command.split('\n')))
    cmd = shlex.split(f'exec -i -u postgres {container_id} psql -U postgres -d {db_name}')
    cmd += sql_command
    if output is None:
        docker_exec_with_output(cmd)
    else:
        with open(output, 'w') as f:
            docker_exec(cmd, stdout=f)


def docker_exec_redis_command(container: str, command: str, output: str = None, append: bool = False):
    container_id = ensure_container_exists(container)
    cmd = shlex.split(f'exec -it {container_id} bash -c "/usr/bin/redis-cli {command}"')
    if output is None:
        docker_exec_with_output(cmd)
    else:
        with open(output, 'a' if append else 'w') as f:
            docker_exec(cmd, stdout=f)


def db_exists(container: str, db_name: str) -> bool:
    container_id = ensure_container_exists(container)
    res = docker_exec_with_output(shlex.split(
        f'exec -i -u postgres {container_id} psql -U postgres -XtAc '
        f'"SELECT 1 FROM pg_database WHERE datname=\'{db_name}\'"'
    ))[0].strip()
    return bool(res)


def _interleave(a: Iterable, b: Iterable) -> Iterable:
    return list(itertools.chain(*zip(a, b)))


def ensure_container_exists(container: str) -> str:
    try:
        container_id = get_containers_id(container)[0]
    except LookupError:
        raise RuntimeError(f'Container {container} is not present on the current node')
    return container_id
