import glob
import shutil
from dataclasses import dataclass
from pathlib import Path
from typing import List, Union

from collect.tasks.collect_task import CollectTask


@dataclass
class CopyItem:
    from_: Union[str, Path]
    to: Path


class CopyTask(CollectTask):
    """Base copy task class.

    Class is used for copying files (by template with *) or dirs to another
        destination.

    Attributes:
        COPY_TASKS: Must be defined in derived classes with a list of CopyItem
            objects, containing the source and destination of the copy.
    """

    COPY_TASKS: List[CopyItem]

    def _run(self) -> None:
        self._run_copy_tasks(self.COPY_TASKS)

    def _run_copy_tasks(self, tasks: List[CopyItem]) -> None:
        assert (
            self._configuration.work_dir is not None
        ), "Configuration work_dir must not be empty for collecting tasks"

        for task in tasks:
            src = task.from_
            dst = self._configuration.work_dir / task.to

            if '*' in str(src):
                copy_method = self._copy_files_by_mask
            else:
                copy_method = self._copy_dir
            copy_method(src, dst)

    def _copy_dir(self, src: Union[str, Path], dst: Path) -> None:
        dst.parent.mkdir(parents=True, exist_ok=True)
        self._logger.info(f'copy: {src}')
        shutil.copytree(str(src), str(dst))

    def _copy_files_by_mask(self, src: Union[str, Path], dst: Path) -> None:
        dst.mkdir(parents=True, exist_ok=True)
        for file in glob.glob(str(src)):
            self._logger.info(f'copy: {file}')
            shutil.copy(file, dst)
