from pathlib import Path
from typing import cast

import requests

import docker_utils
from collect.helpers.utils import ensure_container_exists, stream_file_from_container
from collect.tasks.collect_task import CollectTask
from docker_utils.service import Service


class DumpPrometheusTask(CollectTask):
    TITLE = 'Dump prometheus metrics'

    MONITORING_PROMETHEUS = 'monitoring_prometheus'
    MONITORING_PROMETHEUS_PORT = 9090

    TARGET_DIR_RELATIVE_PATH = Path('dumps/prometheus/')
    SOURCE_DUMP_DIR = Path('/data/snapshots')

    def _check_if_task_should_run(self) -> bool:
        if self._configuration.no_prometheus or not self._has_prometheus_service():
            self._configuration.no_prometheus = True
        return not self._configuration.no_prometheus

    def _run(self):
        self._copy_prometheus_dump(self._dump_prometheus_metrics())

    def _dump_prometheus_metrics(self) -> str:
        resp = requests.post(
            f'http://localhost:{self.MONITORING_PROMETHEUS_PORT}/api/v1/admin/tsdb/snapshot',
            verify=False,
        )
        resp.raise_for_status()
        return resp.json()['data']['name']

    def _copy_prometheus_dump(self, prometheus_dump_name: str):
        dump_archive_name = f'{prometheus_dump_name}.tar.gz'

        dst = cast(Path, self._configuration.work_dir) / self.TARGET_DIR_RELATIVE_PATH / dump_archive_name

        src = self.SOURCE_DUMP_DIR / prometheus_dump_name
        archive = self.SOURCE_DUMP_DIR / dump_archive_name

        prometheus_container_id = docker_utils.container.get_containers_id(self.MONITORING_PROMETHEUS)[0]

        dst.parent.mkdir(parents=True, exist_ok=True)
        docker_utils.docker_exec_with_output(
            ['exec', prometheus_container_id, 'bash', '-c', f'tar -czvf {archive} {src}']
        )
        stream_file_from_container(self.MONITORING_PROMETHEUS, str(archive), str(dst))
        docker_utils.docker_exec_with_output(['exec', prometheus_container_id, 'bash', '-c', f'rm -rf {src}'])
        docker_utils.docker_exec_with_output(['exec', prometheus_container_id, 'bash', '-c', f'rm -rf {archive}'])

    def _has_prometheus_service(self) -> bool:
        container_di = None
        try:
            container_di = ensure_container_exists(self.MONITORING_PROMETHEUS)
        except RuntimeError:
            service_names = Service.get_service_names_by_regex('', fr'.*{self.MONITORING_PROMETHEUS}.*')
            if not service_names:
                self._logger.info('No Prometheus service is found on the cluster, skip step')
            else:
                prometheus_nodes = set(Service.get_service_nodes(service_names[0]))
                self._logger.warning(
                    f'No Prometheus container ({self.MONITORING_PROMETHEUS}) found on the current node. '
                    'If you need the Prometheus database dump, run kata-collect on the following nodes: '
                    f'{prometheus_nodes}.\n'
                )
        return container_di is not None
