from pathlib import Path
from subprocess import CalledProcessError

import docker_utils.container as container_lib
from base_installer.constants.configuration import CEPH_CLUSTER_USER
from base_installer.tasks.helpers.subprocess_runner import SubprocessRunner
from collect.tasks.collect_task import CollectTask


class GetCephDiagnosticTask(CollectTask):
    TITLE = 'Get ceph diagnostic'

    OSD_CONTAINER_PREFIX = 'osd'
    CEPH_DIAGNOSTIC_DIR = Path('diagnostic/ceph/')

    CEPH_COMMAND_FILE_MAP = [
        {
            'cmd': f'ceph df --name client.{CEPH_CLUSTER_USER}',
            'file': 'ceph_df',
        },
        {
            'cmd': f'ceph osd df --name client.{CEPH_CLUSTER_USER}',
            'file': 'ceph_osd_df',
        },
        {
            'cmd': f'ceph osd tree --name client.{CEPH_CLUSTER_USER}',
            'file': 'ceph_osd_tree',
        },
        {
            'cmd': f'ceph osd dump --name client.{CEPH_CLUSTER_USER}',
            'file': 'ceph_osd_dump',
        },
        {
            'cmd': f'ceph mon stat --name client.{CEPH_CLUSTER_USER}',
            'file': 'ceph_mon_stat',
        },
        {
            'cmd': f'ceph quorum_status --name client.{CEPH_CLUSTER_USER}',
            'file': 'ceph_quorum_status',
        },
    ]

    def _run(self):
        dst_dir = self._configuration.work_dir / self.CEPH_DIAGNOSTIC_DIR
        dst_dir.mkdir(parents=True, exist_ok=True)
        for cmd_and_file in self.CEPH_COMMAND_FILE_MAP:
            try:
                output = SubprocessRunner.run(cmd_and_file['cmd'])  # handle error
                with open(dst_dir / cmd_and_file['file'], 'w') as file:
                    file.write(output)
            except CalledProcessError:
                self._logger.warning('Ceph is not found on the current node, skip step')
        self._dump_osd_logs()

    def _dump_osd_logs(self):
        container_names = container_lib.get_containers_names(include_stopped=True)
        osd_container_names = [name for name in container_names if name.startswith(self.OSD_CONTAINER_PREFIX)]
        dst_dir = self._configuration.work_dir / self.CEPH_DIAGNOSTIC_DIR / 'osd_logs'
        dst_dir.mkdir(parents=True, exist_ok=True)

        for name in osd_container_names:
            self._logger.info(f'dumping osd logs for {name}')
            container = container_lib.Container(name, settings=container_lib.ContainerSettings(image=''))
            log_path = dst_dir / f'{name}.log'
            try:
                container.dump_logs_to_file(log_path)
            except Exception:
                self._logger.warning(f'failed to collect osd logs for {name}')
