import shlex
from pathlib import Path

import docker_utils
from collect.tasks.collect_task import CollectTask
from docker_utils import container
from docker_utils.common import split_lines


class GetDockerDiagnosticTask(CollectTask):
    TITLE = 'Get docker diagnostic'

    DOCKER_DIAGNOSTIC_DIR = Path('diagnostic/docker/')
    DOCKER_CONTAINER_LIST_FILE = 'docker_ps'
    DOCKER_NETWORK_INSPECT_FILE = 'network_inspect'
    DOCKER_SERVICE_LIST_FILE = 'service_ls'
    DOCKER_SERVICE_DETAILS_LIST_FILE = 'service_ps'
    DOCKER_VOLUME_LIST_FILE = 'volume_ls'
    DOCKER_VOLUME_INSPECT_FILE = 'volume_inspect'
    DOCKER_DETAILED_CONTAINER_INFO = 'detailed_container_info'

    def _run(self):
        dst_dir = self._configuration.work_dir / self.DOCKER_DIAGNOSTIC_DIR
        dst_dir.mkdir(parents=True, exist_ok=True)

        self._save_docker_containers_info(dst_dir)
        self._save_docker_networks_info(dst_dir)
        self._save_docker_services_info(dst_dir)
        self._save_docker_services_details_info(dst_dir)
        self._save_docker_volumes_info(dst_dir)
        self._save_docker_volume_inspect(dst_dir)
        self._save_ps_auxef_per_container(dst_dir / self.DOCKER_DETAILED_CONTAINER_INFO)

    def _save_docker_diagnostic(self, cmd: str, dst_file: Path):
        output, err = docker_utils.docker_exec_with_output(shlex.split(cmd))
        with open(dst_file, 'w') as file:
            file.write(output or err)

    def _save_docker_containers_info(self, dst_dir: Path):
        self._save_docker_diagnostic(
            cmd='ps -a',
            dst_file=dst_dir / self.DOCKER_CONTAINER_LIST_FILE,
        )

    def _save_docker_services_info(self, dst_dir: Path):
        self._save_docker_diagnostic(
            cmd='service ls',
            dst_file=dst_dir / self.DOCKER_SERVICE_LIST_FILE,
        )

    def _save_docker_services_details_info(self, dst_dir: Path):
        output, _ = docker_utils.docker_exec_with_output(['service', 'ls', '-q'])
        service_ids = output.replace('\n', ' ')

        self._save_docker_diagnostic(
            cmd=f'service ps {service_ids} --no-trunc',
            dst_file=dst_dir / self.DOCKER_SERVICE_DETAILS_LIST_FILE,
        )

    def _save_docker_networks_info(self, dst_dir: Path):
        output, _ = docker_utils.docker_exec_with_output(['network', 'ls', '-q'])
        network_ids = output.replace('\n', ' ')

        self._save_docker_diagnostic(
            cmd=f'network inspect {network_ids}',
            dst_file=dst_dir / self.DOCKER_NETWORK_INSPECT_FILE,
        )

    def _save_docker_volumes_info(self, dst_dir: Path):
        self._save_docker_diagnostic(
            cmd='volume ls',
            dst_file=dst_dir / self.DOCKER_VOLUME_LIST_FILE
        )

    def _save_docker_volume_inspect(self, dst_dir: Path):
        output, _ = docker_utils.docker_exec_with_output(['volume', 'ls'])
        volumes = ' '.join([x.split()[1].strip() for x in split_lines(output)[1:]])

        self._save_docker_diagnostic(
            cmd=f'volume inspect {volumes}',
            dst_file=dst_dir / self.DOCKER_VOLUME_INSPECT_FILE
        )

    def _save_ps_auxef_per_container(self, dst_dir: Path):
        dst_dir.mkdir(parents=True, exist_ok=True)
        for container_name in container.get_containers_names():
            container_id = container.get_containers_id(container_name)
            exec_command = f'exec {container_id[0]} ps auxef'
            try:
                output, err = docker_utils.docker_exec_with_output(shlex.split(exec_command))
                dst_file = dst_dir / f'ps_auxef_{container_name}'
                with open(dst_file, 'w') as file:
                    file.write(output or err)
            except Exception as ex:
                self._logger.error(f'Collect error: {ex}, skip step')
